@@ -25,7 +25,7 @@ const std::string galera::ReplicatorSMM::Param::key_format =
 const std::string galera::ReplicatorSMM::Param::max_write_set_size =
     common_prefix + "max_ws_size";
 
-int const galera::ReplicatorSMM::MAX_PROTO_VER(11);
+int const galera::ReplicatorSMM::MAX_PROTO_VER(9);
 
 galera::ReplicatorSMM::Defaults::Defaults() : map_()
 {
@@ -60,12 +60,6 @@ galera::ReplicatorSMM::InitConfig::InitConfig(gu::Config&       conf,
             conf.add(i->first, i->second);
     }
 
-    conf.set_flags(Param::causal_read_timeout, gu::Config::Flag::type_duration);
-    conf.set_flags(Param::max_write_set_size, gu::Config::Flag::type_integer);
-    conf.set_flags(Param::base_dir, gu::Config::Flag::read_only);
-    conf.set_flags(Param::base_port, gu::Config::Flag::read_only |
-                   gu::Config::Flag::type_integer);
-
     // what is would be a better protection?
     int const pv(gu::from_string<int>(conf.get(Param::proto_max)));
     if (pv > MAX_PROTO_VER)
@@ -75,7 +69,8 @@ galera::ReplicatorSMM::InitConfig::InitConfig(gu::Config&       conf,
         conf.add(Param::proto_max, gu::to_string(MAX_PROTO_VER));
     }
 
-    conf.add(COMMON_BASE_HOST_KEY, gu::Config::Flag::read_only);
+    conf.add(COMMON_BASE_HOST_KEY);
+    conf.add(COMMON_BASE_PORT_KEY);
 
     if (node_address && strlen(node_address) > 0)
     {
@@ -129,7 +124,7 @@ galera::ReplicatorSMM::ParseOptions::ParseOptions(Replicator&       repl,
                                                   gu::Config&       conf,
                                                   const char* const opts)
 {
-    if (opts) conf.parse(opts);
+    conf.parse(opts);
 
     if (conf.get<bool>(Replicator::Param::debug_log))
     {
@@ -202,15 +197,7 @@ galera::ReplicatorSMM::param_set (const std::string& key,
 {
     try
     {
-#ifdef GALERA_HAVE_SSL
-        // Param `socket.ssl_reload` is used as trigger. We should
-        // skip checking if value is changed.
-        if (key != gu::conf::ssl_reload) {
-#endif
         if (config_.get(key) == value) return;
-#ifdef GALERA_HAVE_SSL
-        }
-#endif
     }
     catch (gu::NotSet&) {}
 
@@ -221,21 +208,19 @@ galera::ReplicatorSMM::param_set (const std::string& key,
     if (defaults.map_.find(key) != defaults.map_.end() ||
         key                     == Param::base_host) // is my key?
     {
-        set_param (key, value);
         found = true;
-        config_.set(key, value);
+        set_param (key, value);
+        config_.set (key, value);
     }
 
+    if (key == Certification::PARAM_LOG_CONFLICTS)
+    {
+        cert_.set_log_conflicts(value);
+        return;
+    }
     // this key might be for another module
     else if (0 != key.find(common_prefix))
     {
-        try
-        {
-            cert_.param_set (key, value);
-            found = true;
-        }
-        catch (gu::NotFound&) {}
-
         try
         {
             gcs_.param_set (key, value);
@@ -249,15 +234,6 @@ galera::ReplicatorSMM::param_set (const std::string& key,
             found = true;
         }
         catch (gu::NotFound&) {}
-
-#ifdef GALERA_HAVE_SSL
-        try
-        {
-            gu::ssl_param_set (key, value, config_);
-            found = true;
-        }
-        catch (gu::NotFound&) {}
-#endif
     }
 
     if (!found) throw gu::NotFound();
