@@ -214,11 +214,6 @@ BOOLEAN _gu_no_db_    = TRUE;	         /* TRUE if no debugging at all      */
  */
 
 
-/*
- * Galera does not provided _sanity which is used when SAFEMALLOC is
- * defined
- */
-#undef SAFEMALLOC
 
 IMPORT int _sanity(const char *file, uint line);
 
@@ -356,7 +351,7 @@ static char *static_strtok(char *s1, char chr);
 
 #undef EXISTS
 #if !defined(HAVE_ACCESS) || defined(MSDOS)
-#define EXISTS(pathname) (FALSE)			   /* Assume no existence */
+#define EXISTS(pathname) (FALSE)			   /* Assume no existance */
 #define Writable(name) (TRUE)
 #else
 #define EXISTS(pathname)	 (access (pathname, F_OK) == 0)
